/* hdbapp_about.c
** hdb about: report metadata in hdb
** wcm, 2010.12.16 - 2010.12.17
** ===
*/

/* libc: */
#include <stdlib.h>

/* unix: */
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>

/* lasagna: */
#include "cstr.h"
#include "buf.h"
#include "ioq.h"
#include "ioq_std.h"
#include "nextopt.h"
#include "nfmt.h"
#include "sysstr.h"
#include "uchar.h"
#include "upak.h"

/* lasagna hdb reader: */
#include "hdb.h"

/* hdbapp: */
#include "hdbapp.h"


#ifndef HDBAPP_MODULE
/* configuration for standalone program: */
/* these globals setup in main(): */
const char *progname = NULL;
const char *usage_prog = NULL;
/* these globals remain NULL in standalone: */
const char *progmode = NULL;
const char *usage_mode = NULL;
/* entry point for standalone: */
#define  HDBAPP_MAIN  main
#else
/* entry point for module: */
#define  HDBAPP_MAIN  hdbapp_about
#endif /* #ifndef HDBAPP_MODULE */

/* additional i/o macro for stdout: */
#define vputs(...) \
  ioq_vputs(ioq1, __VA_ARGS__)


/* objects in local scope: */
/* -a: "all" */
static int opt_a = 0;
/* -c: "comment" */
static int opt_c = 0;
/* -i: "ident string" */
static int opt_i = 0;
/* -n: "number of records" */
static int opt_n = 0;


/* functions in local scope: */
static int do_about(int fd);

/* do_about()
**  write requested info to stdout
**  abort of failure
**
**  note: does not use hdb.h api
**  (in order to inspect/report unsupported identifier)
*/
static
int
do_about(int fd)
{
  uchar_t   header[HDB_CBASE];
  uchar_t  *buf;
  char      idstr[HDB_IDLEN + 1];
  char      nfmt[NFMT_SIZE];
  uint32_t  nrecs = 0;
  uint32_t  rbase = 0;
  ssize_t   r;

  /* suck in file header: */
  if((r = read(fd, header, sizeof header)) == -1){
      fatal_syserr("failure reading file header");
  }
  if(r != (ssize_t)(sizeof header)){
      fatal(100, "short file header");
  }

  /* build idstr: */
  buf_copy(idstr, header, HDB_IDLEN);
  idstr[HDB_IDLEN] = '\0';

  /* check identifier prefix match: */
  if(buf_cmp(idstr, "hdb32/", cstr_len("hdb32/")) != 0){
      fatal(100, "unknown file type");
  }

  /* display identifier found if prefix match: */
  if(opt_i){
      vputs("identifier: ", idstr, "\n");
  }

  /* to continue, require supported identifier: */
  if(buf_cmp(idstr, HDB32_IDENT, HDB_IDLEN) != 0){
      fatal(100, "identifier \"", idstr ,"\" not supported");
  }

  /* record count: */
  buf = header + HDB_IDLEN;
  nrecs = upak32_UNPACK(buf);
  buf += 4;
  rbase = upak32_UNPACK(buf); 

  if(opt_n){
      vputs("records: ", nfmt_uint32(nfmt, nrecs), "\n");
  }

  /* comment: */
  if(opt_c){
      uint32_t  clen = rbase - HDB_CBASE;
      uchar_t   cbuf[512];
      size_t    n;

      vputs("comment: ");
      if(clen == 0){
          vputs("(none)\n");
      } else {
          while(clen > 0){
              n = sizeof cbuf;
              if(n > clen) n = clen;
              r = read(fd, cbuf, n);
              switch(r){
              case -1: fatal_syserr("failure reading comment area"); break;
              case  0: fatal(100, "unexpected eof"); break;
              }
              ioq_put(ioq1, cbuf, r);
              clen -= r;
          }
          vputs("\n");
      }
  }

  ioq_flush(ioq1);

  return 0;
}



int
HDBAPP_MAIN(int argc, char *argv[])
{
  nextopt_t    nopt;
  char         opt;
  int          fd;
  int          is_stdin = 0;
  int          e;

#ifndef HDBAPP_MODULE
/* standalone options/initialization: */
  usage_prog = "[-hV] [-acin] [hdb]";
  nextopt_init(&nopt, argc, argv, ":hVacin");
  progname = nextopt_progname(&nopt);
#else
/* module options/initialization: */
  usage_mode = "[-h] [-acin] [hdb]";
  nextopt_init(&nopt, argc, argv, ":hacin");
#endif

  while((opt = nextopt(&nopt))){
      char optc[2] = {nopt.opt_got, '\0'};
      switch(opt){
      case 'h': usage(); die(0); break;
      case 'V': version(); die(0); break;
      case 'a': ++opt_a; break;
      case 'c': ++opt_c; break;
      case 'i': ++opt_i; break;
      case 'n': ++opt_n; break;
      case ':':
          fatal_usage("missing argument for option -", optc);
          break;
      case '?':
          if(nopt.opt_got != '?'){
              fatal_usage("invalid option -", optc);
          }
          /* else fallthrough: */
      default:
          die_usage(); break;
      }
  }

  argc -= nopt.arg_ndx;
  argv += nopt.arg_ndx;

  if((argc > 0) && (argv[0] != NULL)){
      /* hdb file argument: */
      if((fd = open(argv[0], O_RDONLY | O_NONBLOCK)) == -1){
          fatal_syserr("failure on open() for ", argv[0]);
      }
  }else{
      /* read from stdin: */
      fd = 0;
      ++is_stdin;
  }

  /* default report: number of records and descriptive comment:  */
  if(!opt_c && !opt_i && !opt_n){++opt_c; ++opt_n;}
  /* setup for report all: */
  if(opt_a){++opt_c; ++opt_i; ++opt_n;}

  /* do_about() returns 0 or aborts on failure: */
  e = do_about(fd);

  /* close/cleanup: */
  if(!is_stdin){
      close(fd);
  }

  return e;
}


/* eof (hdbapp_about.c) */
